import enum
import os
import pathlib
import subprocess
import sys

from collections import namedtuple

FailedWells = namedtuple("FailedWells", ["pre_run", "post_run"])


class DropState(enum.Enum):
    OK = "ok"
    FAULT = "fault"


class LogFiles(namedtuple("_LogFiles", ["folder", "print", "drops"])):
    __slots__ = ()

    def __bool__(self):
        if self.print and self.drops:
            return True
        else:
            return False


def _find_files(dir_path, endswith):
    visible = (i for i in dir_path.iterdir() if not i.name.startswith("."))
    return [item for item in visible if item.name.endswith(endswith)]


def find_log_files(folder):
    dir_path = pathlib.Path(folder)
    tmp_print_log = _find_files(dir_path, "_Logfile.log")
    if len(tmp_print_log) == 1:
        print_log = tmp_print_log[0]
    else:
        print_log = None
    drop_logs = _find_files(dir_path, ".cor")
    return LogFiles(dir_path, print_log, drop_logs)


def get_failed_drop_checks(dataframe, measurement, nozzle=None):
    if nozzle is not None:
        selection = dataframe["nozzle"] == nozzle
        nozzle_df = dataframe[selection]
    else:
        nozzle_df = dataframe
    # select first only the failed rows
    selection = nozzle_df["status"] == DropState.FAULT.value
    failure_df = nozzle_df[selection]
    # selection based on measurement type
    selection = failure_df["measurement"] == measurement
    return failure_df[selection]


def find_failed_drops(dataframe, nozzle=None):
    pre_run_df = get_failed_drop_checks(dataframe, "pre run", nozzle)
    all_post_run_df = get_failed_drop_checks(dataframe, "post run", nozzle)
    # if a check already failed in the pre run, we exclude it from the post run
    selection = all_post_run_df["well_id"].isin(pre_run_df["well_id"])
    post_run_df = all_post_run_df[~selection]
    return FailedWells(pre_run_df, post_run_df)


def open_with_default_app(some_path):
    if sys.platform.startswith("linux"):
        subprocess.call(["xdg-open", some_path])
    elif sys.platform.startswith("darwin"):
        subprocess.call(["open", some_path])
    elif sys.platform.startswith("win"):
        os.startfile(some_path)
